from koodous.representations.base import BaseRepresentation


class Rule(BaseRepresentation):
    """The rule representation.
    """

    def __init__(self):
        raise NotImplementedError()  # TODO
